// Program written by Sameer Shelke (IITBombay)
// Explicit designing of a IIR Low pass Chebyshev filter

////////// Calculate Filter Order /////////////////////////////////
//Function is used to calculate order of the analog filter.
funcprot(0)
function N = FilterOrder(RippleFactor1, RippleFactor2,PassBandFreq, StopBandFreq)
Temp = sqrt(RippleFactor2/RippleFactor1); 
N = ceil(acosh(Temp)/acosh(StopBandFreq/PassBandFreq)); 
endfunction
///////////////////////////////////////////////////////////////////
 
////////////////Frequency Transformation //////////////////////////
//Function is used to normalized digital filter specification frequencies and
// transforming it to the analog frequncies
 
function TranFreq = FreqTrans(PassBandFreq1,PassBandFreq2,StopBandFreq1,StopBandFreq2,SamplingFreq)
  TranFreq(1) = tan((%pi*PassBandFreq1)/SamplingFreq);
  TranFreq(2) = tan((%pi*PassBandFreq2)/SamplingFreq);
  TranFreq(3) = tan((%pi*StopBandFreq1)/SamplingFreq); 
  TranFreq(4) = tan((%pi*StopBandFreq2)/SamplingFreq);  
endfunction
////////////////////////////////////////////////////////////////
 
///////////////////// Stop band calculation ////////////////////
////// function is used to calculate stop band of analog filter.
function SBFreq = CalSBFreq(SBHighFreq,SBLowFreq,CutOffFreq,BandWidth)
  Temp1 = (((SBHighFreq)^2 - (CutOffFreq)^2)/(BW * SBHighFreq)); 
  Temp2 = (((SBLowFreq)^2 - (CutOffFreq)^2)/(BW * SBLowFreq));
  SBFreq = min(abs(Temp1),abs(Temp2));
endfunction
///////////////////////////////////////////////////////////////////////////////////////
 
//====Low Pass Chebyshev filter design using Bilinear Transformation method =/ 
/////////// Digital Filter Design Specification //////////////////
PassBandHighFreq = 3100;
PassBandLowFreq =  3000;
StopBandHighFreq = 3500;
StopBandLowFreq =  3400;
SampleRate = 8000;
PassBandTol = 0.1;
StopBandTol = 0.1;
///////////////////////////////////////////////////////////////////
 
clc;
//// Step 1: Find Out the Analaog frequencies from given digital frequencies
//// AnaFreq(1) = Analog Pass Band High Frequncy
//// AnaFreq(2) = Analog Pass Band Low Frequency
///  AnaFreq(3) = Analog Stop Band High Frequncy
///  AnaFreq(3) = Analog Stop Band Low Frequncy
 
AnaFreq = FreqTrans(PassBandHighFreq,PassBandLowFreq,StopBandHighFreq,StopBandLowFreq,SampleRate);
 
//// Step 2: Find Out the Bandwidth of the analog filter
BW = abs(AnaFreq(1)- AnaFreq(2));           // AnaFreq1= Pass Band High Frequency, AnaFreq2 = PassBand Low Frequency
 
//// Step 3: Find Out Cut Off frequncy of Analog filter
CutOffFreq = sqrt(AnaFreq(1) * AnaFreq(2));
 
//// Step 4: Assume Pass Band Frequncy =1 and Calculate Stop Band Frequency
AnaPBFreq = 1;
AnaSBFreq = CalSBFreq(AnaFreq(3),AnaFreq(4),CutOffFreq,BW);
 
//// Step 5: Find the order of the filter
D1=(1/(1-PassBandTol)^2)-1;
D2=(1/StopBandTol^2)-1;
N = FilterOrder(D1,D2,AnaPBFreq,AnaSBFreq);
 
//// Step 6: Calculate chebyshev filter coeffiecients ////////////////////////
for Index = 1:N
  A(Index)   = ((2*(Index-1)+1)*%pi)/(2*N);
  B(Index)   =  (asinh(1/(sqrt (D1))))/N;
  SLK(Index) =  AnaPBFreq*sin(A(Index))*sinh(B(Index))+%i*AnaPBFreq*cos(A(Index))*cosh(B(Index));
end
 
//// Step 7: Implement Analog Transfer function
e = sqrt(D1);
 if N/2 == ceil(N/2)
    Ko = (1/sqrt(1+e^2))*prod(-SLK); //find kapa Ko
 else
    Ko = prod(-SLK);
 end;
Deno = poly(SLK,"s");
AnaFilterTF =[real(Ko)/real(Deno)];
disp("====Transfer Function of Analog Filter======");
disp(AnaFilterTF);
 
//// Step 9: Perform bilinear transformation ////////////////////
//// Substitute s with ((z-1)/(z+1)) in analog transfer function
z = poly(0,"z");
DisFilterTF =horner(AnaFilterTF,((z-1)/(z+1)));
disp("====Transfer Function of Digital Filter=======");
disp(DisFilterTF);
 
//// Step 10: Plot frequency Response //////////////////////////
[HZ,Fr] = frmag(DisFilterTF,256);
xtitle(" Frequency Response of Chebyshev Filter ");
xlabel(" Frequency (rad/sec) ");
plot(Fr,abs(HZ));